from enum import Enum, unique


@unique
class BaseEnum(str, Enum):
    def __str__(self) -> str:
        return str(self.value)

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}.{self.name}>"

    @classmethod
    def contains(cls, value: str) -> bool:
        return value.upper() in cls.__members__


class InputType(BaseEnum):
    API_CALL = "api_call"
    ATOM = "atom"
    CLASS_CREATED = "class_created"
    CLASS_SEARCHED = "class_searched"
    CONNECTION_COUNTRY = "connection_country"
    CONNECTION_IP = "connection_ip"
    CONSOLE_OUTPUT = "console_output"
    CREATED_SERVICE = "created_service"
    DESKTOP = "desktop"
    DNS_LOOKUP = "dns_lookup"
    DOTNET_CLASS = "dotnet_class"
    DOTNET_METHOD = "dotnet_method"
    DOTNET_TYPELIB = "dotnet_typelib"
    ENVIRONMENT_QUERY = "environment_query"
    ENVIRONMENT_WRITE = "environment_write"
    EVENT = "event"
    EXECUTED_COMMAND = "executed_command"
    EXECUTED_COMMAND_NORMALIZED = "executed_command_normalized"
    FILE_ACCESS = "file_access"
    FILE_DELETE = "file_delete"
    FILE_READ = "file_read"
    FILE_WRITE = "file_write"
    HTTP_GET = "http_get"
    HTTP_POST = "http_post"
    IRC_COMMAND = "irc_command"
    JOB = "job"
    KEY_ACCESS = "key_access"
    KEY_DELETE = "key_delete"
    KEY_READ = "key_read"
    KEY_WRITE = "key_write"
    MAILSLOT = "mailslot"
    MODIFIED_CLIPBOARD = "modified_clipboard"
    MUTEX = "mutex"
    PE_ENTRY_POINT = "pe_entry_point"
    PE_ICONHASH = "pe_iconhash"
    PE_IMPHASH = "pe_imphash"
    PE_PDB = "pe_pdb"
    PE_RICH_HEADER = "pe_rich_header"
    PE_SECTION = "pe_section"
    PE_SIGNER = "pe_signer"
    PE_SIGNER_THUMBPRINT = "pe_signer_thumbprint"
    PIPE = "pipe"
    RESOLVED_API = "resolved_api"
    SCHEDULED_TASK = "scheduled_task"
    SECTION = "section"
    SEMAPHORE = "semaphore"
    SIGNATURE = "signature"
    STARTED_SERVICE = "started_service"
    SURICATA = "suricata"
    TIMER = "timer"
    USER_AGENT = "user_agent"
    WINDOW_CREATED = "window_created"
    WINDOW_SEARCHED = "window_searched"
    WMI_OBJECT = "wmi_object"


class NamedObjectStrictType(BaseEnum):
    """
    Enum represents the named objects types that should be quoted with (^|\\) and $.
    Api_calls and resolved_apis are not in this list.
    """

    ATOM = "atom"
    CLASS_CREATED = "class_created"
    CLASS_SEARCHED = "class_searched"
    CONNECTION_COUNTRY = "connection_country"
    CONNECTION_IP = "connection_ip"
    CONSOLE_OUTPUT = "console_output"
    CREATED_SERVICE = "created_service"
    DESKTOP = "desktop"
    DNS_LOOKUP = "dns_lookup"
    DOTNET_CLASS = "dotnet_class"
    DOTNET_METHOD = "dotnet_method"
    DOTNET_TYPELIB = "dotnet_typelib"
    ENVIRONMENT_QUERY = "environment_query"
    ENVIRONMENT_WRITE = "environment_write"
    EVENT = "event"
    EXECUTED_COMMAND = "executed_command"
    EXECUTED_COMMAND_NORMALIZED = "executed_command_normalized"
    FILE_ACCESS = "file_access"
    FILE_DELETE = "file_delete"
    FILE_READ = "file_read"
    FILE_WRITE = "file_write"
    HTTP_GET = "http_get"
    HTTP_POST = "http_post"
    IRC_COMMAND = "irc_command"
    JOB = "job"
    KEY_ACCESS = "key_access"
    KEY_DELETE = "key_delete"
    KEY_READ = "key_read"
    KEY_WRITE = "key_write"
    MAILSLOT = "mailslot"
    MODIFIED_CLIPBOARD = "modified_clipboard"
    MUTEX = "mutex"
    PE_ENTRY_POINT = "pe_entry_point"
    PE_ICONHASH = "pe_iconhash"
    PE_IMPHASH = "pe_imphash"
    PE_PDB = "pe_pdb"
    PE_RICH_HEADER = "pe_rich_header"
    PE_SECTION = "pe_section"
    PE_SIGNER = "pe_signer"
    PE_SIGNER_THUMBPRINT = "pe_signer_thumbprint"
    PIPE = "pipe"
    SCHEDULED_TASK = "scheduled_task"
    SECTION = "section"
    SEMAPHORE = "semaphore"
    SIGNATURE = "signature"
    STARTED_SERVICE = "started_service"
    SURICATA = "suricata"
    TIMER = "timer"
    USER_AGENT = "user_agent"
    WINDOW_CREATED = "window_created"
    WINDOW_SEARCHED = "window_searched"
    WMI_OBJECT = "wmi_object"
